'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _adapter = require('./adapter');

var _adapter2 = _interopRequireDefault(_adapter);

var _serializer = require('./serializer');

var _serializer2 = _interopRequireDefault(_serializer);

var _repository = require('./repository');

var _repository2 = _interopRequireDefault(_repository);

var _migrator = require('./migrator');

var _migrator2 = _interopRequireDefault(_migrator);

var _dropper = require('./dropper');

var _dropper2 = _interopRequireDefault(_dropper);

var _syncer = require('./syncer');

var _syncer2 = _interopRequireDefault(_syncer);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = (0, _debug2.default)('postgresql');

/**
 * PostgresqlFacade
 *
 * @class PostgresqlFacade
 */

var PostgresqlFacade = function () {

  /**
   * Create a new PostgresqlFacade instance
   *
   * @param {Object} config
   */

  function PostgresqlFacade(config) {
    _classCallCheck(this, PostgresqlFacade);

    debug('-> constructor', config);
    this.adapter = new _adapter2.default(config);
    this.serializer = new _serializer2.default();
    this.repository = new _repository2.default(this.serializer, this.adapter);
    this.migrator = new _migrator2.default(this.adapter, config);
    this.dropper = new _dropper2.default(config);
    this.syncer = new _syncer2.default(config, this.adapter, this.serializer);
  }

  /**
   * Insert / update item.
   * @param  {Item} item          The item to store
   * @param  {Schema} schema      The schema for the name and primary properties.
   * @param  {Metadata} metadata  An object with a projectName and projectVersion property.
   * @return {Promise}
   */

  _createClass(PostgresqlFacade, [{
    key: 'putItem',
    value: function putItem(item, schema, metadata) {
      debug('-> putItem');
      return this.repository.putItem(item, schema, metadata);
    }

    /**
     * Remove an item.
     * @param  {String}   itemId      The id of the item
     * @param  {Schema}   schema      The schema for the name and primary properties.
     * @param  {Metadata} metadata    An object with a projectName and projectVersion property.
     * @return {Promise}
     */

  }, {
    key: 'delItem',
    value: function delItem(itemId, schema, metadata) {
      debug('-> delItem');
      return this.repository.delItem(itemId, schema, metadata);
    }

    /**
     * Create a new postgresql database and it's tables.
     *
     * @param  {String}   projectId
     * @param  {Project}  oldProject
     * @param  {Project}  project
     * @param  {Array}    changes
     *
     * @return {Promise}
     */

  }, {
    key: 'migrate',
    value: function migrate(projectId, oldProject, project, changes) {
      var _this = this;

      debug('-> migrate');
      return this.migrator.migrate(projectId, oldProject, project, changes).then(function () {
        if (project.version > 1) {
          return _this.syncer.sync(projectId, oldProject, project, changes);
        }
      });
    }

    /**
     * Drop postgresql database.
     *
     * @param  {String} projectId
     * @param  {Number} projectVersion
     *
     * @return {Promise}
     */

  }, {
    key: 'drop',
    value: function drop(projectId, projectVersion) {
      debug('-> drop');
      return this.dropper.drop(projectId, projectVersion);
    }
  }]);

  return PostgresqlFacade;
}();

exports.default = PostgresqlFacade;