'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash.contains');

var _lodash2 = _interopRequireDefault(_lodash);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _pg = require('pg');

var _pg2 = _interopRequireDefault(_pg);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

_bluebird2.default.promisifyAll(_pg2.default);

var debug = (0, _debug2.default)('postgresql.migrator');

var VERSION_SEPERATOR = '@';

var capitalizeFirstLetter = function capitalizeFirstLetter(string) {
  return string.charAt(0).toUpperCase() + string.slice(1);
};

/**
 * PostgresqlMigrator
 *
 * @class PostgresqlMigrator
 */

var PostgresqlMigrator = function () {
  /**
   * @param {PostgresqlAdapter} adapter
   */

  function PostgresqlMigrator(adapter, config) {
    _classCallCheck(this, PostgresqlMigrator);

    this.adapter = adapter;
    this.config = config;
  }

  /**
   * Create a new postgresql database and it's tables.
   * @param  {String} projectId
   * @param  {Object} oldProject
   * @param  {Object} project
   * @param  {Array} changes
   *
   * @return {Promise}
   */

  _createClass(PostgresqlMigrator, [{
    key: 'migrate',
    value: function migrate(projectId, oldProject, project, changes) {
      var connectionString, client;
      return regeneratorRuntime.async(function migrate$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            connectionString = 'postgresql://' + this.config.user + ':' + this.config.password + '@' + this.config.host + '/';
            _context.next = 3;
            return regeneratorRuntime.awrap(this.checkExtensions(connectionString));

          case 3:
            _context.next = 5;
            return regeneratorRuntime.awrap(this.createDatabase(connectionString, project.name, project.version));

          case 5:
            _context.next = 7;
            return regeneratorRuntime.awrap(this.adapter.getConnection(project.name, project.version));

          case 7:
            client = _context.sent;
            _context.next = 10;
            return regeneratorRuntime.awrap(this.createTables(client, project.schemas));

          case 10:
            return _context.abrupt('return', {
              success: true
            });

          case 11:
          case 'end':
            return _context.stop();
        }
      }, null, this);
    }
  }, {
    key: 'checkExtensions',
    value: function checkExtensions(connectionString) {
      return _pg2.default.connectAsync(connectionString + 'template_postgis').spread(function (postgisTemplateConnection, done) {
        return postgisTemplateConnection.queryAsync('SELECT extname, extversion from pg_extension').then(function (result) {
          var promise = _bluebird2.default.resolve();
          var extensions = result.rows.map(function (row) {
            return row.extname;
          });
          if (!(0, _lodash2.default)(extensions, 'postgis')) {
            debug('"postgis" extension missing, installing it.');
            promise = promise.then(function () {
              return postgisTemplateConnection.queryAsync('CREATE EXTENSION postgis');
            });
          }
          if (!(0, _lodash2.default)(extensions, 'uuid-ossp')) {
            debug('"uuid-ossp" extension missing, installing it.');
            promise = promise.then(function () {
              return postgisTemplateConnection.queryAsync('CREATE EXTENSION "uuid-ossp"');
            });
          }
          return promise;
        }).finally(function () {
          return done();
        });
      });
    }

    /**
     * Creates a new database for given projectId and project version.
     *
     * @param  {String} connectionString The postgresql connectionString
     * @param  {String} projectName      The project name
     * @param  {Number} projectVersion   The project version
     */

  }, {
    key: 'createDatabase',
    value: function createDatabase(connectionString, projectName, projectVersion) {
      return _pg2.default.connectAsync(connectionString + 'template_postgis').spread(function (postgresConnection, done) {
        return postgresConnection.queryAsync('CREATE DATABASE "' + projectName + VERSION_SEPERATOR + projectVersion + '" TEMPLATE=template_postgis').catch(function (err) {
          console.error('Error while creating database', err);
          throw err;
        }).finally(function () {
          return done();
        });
      });
    }

    /**
     * Creates tables in database based on the given blueprints.
     *
     * @param  {PostgresqlClient}  client
     * @param  {Object}            schemas
     */

  }, {
    key: 'createTables',
    value: function createTables(client, schemas) {
      var _this = this;

      return _bluebird2.default.map(Object.keys(schemas), function (schemaId) {
        var schema = schemas[schemaId];
        return client.schema.createTable(schema.name, function (table) {
          Object.keys(schema.columns).forEach(function (columnId) {
            var column = schema.columns[columnId];
            var chain = _this['define' + capitalizeFirstLetter(column.type).replace('[]', 'Array')](table, column.name);
            if (schema.primary === columnId) {
              chain.primary();
            }
            if (column.unique) {
              chain.unique();
            }
            if (column.references) {
              chain.inTable(schemas[column.references.schemaId].name).references(schemas[column.references.schemaId].columns[column.references.columnId].name);
            }
            if (column.unsigned) {
              chain.unsigned();
            }
            if (column.nullable === false) {
              chain.notNullable();
            }
            if (column.comment) {
              chain.comment(column.comment);
            }
          });
        }).then(function (res) {
          return res;
        }, function (err) {
          console.error('error while creating tables', err);
        });
      });
    }

    /**
     * Defines an INTEGER column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineInteger',
    value: function defineInteger(table, columnName) {
      return table.integer(columnName);
    }

    /**
     * Defines an INTEGER[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineIntegerArray',
    value: function defineIntegerArray(table, columnName) {
      return table.specificType(columnName, 'INTEGER[]');
    }

    /**
     * Defines an UUID column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineUuid',
    value: function defineUuid(table, columnName) {
      return table.uuid(columnName);
    }

    /**
     * Defines an UUID[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineUuidArray',
    value: function defineUuidArray(table, columnName) {
      return table.specificType(columnName, 'UUID[]');
    }

    /**
     * Defines an STRING column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineString',
    value: function defineString(table, columnName) {
      return table.string(columnName);
    }

    /**
     * Defines an STRING[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineStringArray',
    value: function defineStringArray(table, columnName) {
      return table.specificType(columnName, 'VARCHAR[]');
    }

    /**
     * Defines an TEXT column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineText',
    value: function defineText(table, columnName) {
      return table.text(columnName);
    }

    /**
     * Defines an TEXT[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineTextArray',
    value: function defineTextArray(table, columnName) {
      return table.specificType(columnName, 'TEXT[]');
    }

    /**
     * Defines an DATETIME column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineDatetime',
    value: function defineDatetime(table, columnName) {
      return table.dateTime(columnName);
    }

    /**
     * Defines an DATETIME[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineDatetimeArray',
    value: function defineDatetimeArray(table, columnName) {
      return table.specificType(columnName, 'TIMESTAMP WITH TIME ZONE[]');
    }

    /**
     * Defines an DATE column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineDate',
    value: function defineDate(table, columnName) {
      return table.date(columnName);
    }

    /**
     * Defines an DATE[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineDateArray',
    value: function defineDateArray(table, columnName) {
      return table.specificType(columnName, 'DATE[]');
    }

    /**
     * Defines an REAL column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineFloat',
    value: function defineFloat(table, columnName) {
      return table.float(columnName);
    }

    /**
     * Defines an REAL[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineFloatArray',
    value: function defineFloatArray(table, columnName) {
      return table.specificType(columnName, 'REAL[]');
    }

    /**
     * Defines an GEOMETRY column of type Point on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'definePoint',
    value: function definePoint(table, columnName) {
      return table.specificType(columnName, 'geometry(Point,4326)');
    }

    /**
     * Defines an GEOMETRY column of type MultiPoint on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'definePointArray',
    value: function definePointArray(table, columnName) {
      return table.specificType(columnName, 'geometry(MultiPoint,4326)');
    }

    /**
     * Defines an GEOMETRY column of type LineString on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineLinestring',
    value: function defineLinestring(table, columnName) {
      return table.specificType(columnName, 'geometry(LineString,4326)');
    }

    /**
     * Defines an GEOMETRY column of type MultiLineString on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineLinestringArray',
    value: function defineLinestringArray(table, columnName) {
      return table.specificType(columnName, 'geometry(MultiLineString,4326)');
    }

    /**
     * Defines an GEOMETRY column of type Polygon on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'definePolygon',
    value: function definePolygon(table, columnName) {
      return table.specificType(columnName, 'geometry(Polygon,4326)');
    }

    /**
     * Defines an GEOMETRY column of type MultiPolygon on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'definePolygonArray',
    value: function definePolygonArray(table, columnName) {
      return table.specificType(columnName, 'geometry(MultiPolygon,4326)');
    }

    /**
     * Defines an BOOLEAN column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineBoolean',
    value: function defineBoolean(table, columnName) {
      return table.boolean(columnName);
    }

    /**
     * Defines an BOOLEAN[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineBooleanArray',
    value: function defineBooleanArray(table, columnName) {
      return table.specificType(columnName, 'BOOLEAN[]');
    }

    /**
     * Defines a JSON column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineJson',
    value: function defineJson(table, columnName) {
      return table.json(columnName);
    }

    /**
     * Defines a JSON[] column on given table.
     *
     * @protected
     * @param  {Object} table     Knex.js table object (given through createTable's callback)
     * @param  {String} columnName Name of the column
     */

  }, {
    key: 'defineJsonArray',
    value: function defineJsonArray(table, columnName) {
      return table.specificType(columnName, 'JSON[]');
    }
  }]);

  return PostgresqlMigrator;
}();

exports.default = PostgresqlMigrator;