'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash.contains');

var _lodash2 = _interopRequireDefault(_lodash);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var knex = require('knex')({ dialect: 'postgres' });

/**
 * PostgresqlRepository
 *
 * @class PostgresqlReposirory
 */

var PostgresqlRepository = function () {
  /**
   * @param {PostgresqlSerializer}  serializer
   * @param {PostgresqlAdapter}     adapter
   */

  function PostgresqlRepository(serializer, adapter) {
    _classCallCheck(this, PostgresqlRepository);

    this.serializer = serializer;
    this.adapter = adapter;
  }

  /**
   * Insert / update item.
   *
   * @param  {Item} item          The item to store
   * @param  {Schema} schema      The schema for the name and primary properties.
   * @param  {Metadata} metadata  An object with a projectName and projectVersion property.
   * @return {Promise}
   */

  _createClass(PostgresqlRepository, [{
    key: 'putItem',
    value: function putItem(item, schema, metadata) {
      var serializedItem, client, exists;
      return regeneratorRuntime.async(function putItem$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            // serialize the item
            serializedItem = this.serializer.serialize(item, schema);
            // retrieve the knex.js client for the given version

            client = this.adapter.getConnection(metadata.projectName, metadata.projectVersion);
            // check if the item exists

            _context.next = 4;
            return regeneratorRuntime.awrap(this.itemExists(client, schema, serializedItem));

          case 4:
            exists = _context.sent;

            if (!exists) {
              _context.next = 11;
              break;
            }

            _context.next = 8;
            return regeneratorRuntime.awrap(this.updateItem(client, schema, serializedItem));

          case 8:
            return _context.abrupt('return', _context.sent);

          case 11:
            _context.next = 13;
            return regeneratorRuntime.awrap(this.insertItem(client, schema, serializedItem));

          case 13:
            return _context.abrupt('return', _context.sent);

          case 14:
          case 'end':
            return _context.stop();
        }
      }, null, this);
    }

    /**
     * Remove an item.
     * @param  {String}   itemId      The id of the item
     * @param  {Schema}   schema      The schema for the name and primary properties.
     * @param  {Metadata} metadata    An object with a projectName and projectVersion property.
     * @return {Promise}
     */

  }, {
    key: 'delItem',
    value: function delItem(itemId, schema, metadata) {
      var client = this.adapter.getConnection(metadata.projectName, metadata.projectVersion);
      var wheres = {};
      var key = schema.columns[schema.primary].name;
      wheres[key] = itemId;
      return client.table(schema.name).where(wheres).del()
      // triggers knex.js's promise strategy
      .then(function (res) {
        return res;
      });
    }

    /**
     * Checks if an item already exists in the database.
     * @protected
     * @param  {PostgresqlClient} client          The knex client
     * @param  {Schema}           schema          Schema options
     * @param  {Item}             item            The serialized item
     * @return {Promise} Promise resolving true or false (exists or doesn't exist).
     */

  }, {
    key: 'itemExists',
    value: function itemExists(client, schema, item) {
      return client.table(schema.name).first().where(this.getWheres(schema, item)).then(function (row) {
        return !!row;
      });
    }

    /**
     * Updates an item in the database.
     * @protected
     * @param  {PostgresqlClient} client            The knex client
     * @param  {Schema}           schema            Schema information
     * @param  {Item}             item              The serialized item
     * @return {Promise}
     */

  }, {
    key: 'updateItem',
    value: function updateItem(client, schema, item) {
      return client.table(schema.name).where(this.getWheres(schema, item)).update(item)
      // triggers knex.js's promise strategy
      .then(function (res) {
        return res;
      });
    }

    /**
     * Inserts a new item into the database.
     * @protected
     * @param  {PostgresqlClient} client          The knex client
     * @param  {Schema}           schema          Schema information
     * @param  {Item}             item            The serialized item
     * @return {Promise}
     */

  }, {
    key: 'insertItem',
    value: function insertItem(client, schema, item) {
      return client.table(schema.name).insert(item)
      // triggers knex.js's promise strategy
      .then(function (res) {
        return res;
      });
    }

    /**
     * Get where query
     * @protected
     * @param  {Schema} schema         Schema information
     * @param  {Item}   item           The serialized item
     * @return {Object}                The query
     */

  }, {
    key: 'getWheres',
    value: function getWheres(schema, item) {
      var wheres = {};
      var key = schema.columns[schema.primary].name;
      wheres[key] = item[key];
      return wheres;
    }
  }]);

  return PostgresqlRepository;
}();

exports.default = PostgresqlRepository;