'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _pg = require('pg');

var _pg2 = _interopRequireDefault(_pg);

var _lodash = require('lodash.contains');

var _lodash2 = _interopRequireDefault(_lodash);

var _through = require('through');

var _through2 = _interopRequireDefault(_through);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _schemaMapperTransformer = require('schema-mapper-transformer');

var _schemaMapperTransformer2 = _interopRequireDefault(_schemaMapperTransformer);

var _pgQueryStream = require('pg-query-stream');

var _pgQueryStream2 = _interopRequireDefault(_pgQueryStream);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

_bluebird2.default.promisifyAll(_pg2.default);
var debug = (0, _debug2.default)('postgresql.syncer');

var VERSION_SEPERATOR = '@';

var PostgresqlSyncer = function () {
  function PostgresqlSyncer(config, adapter, serializer) {
    _classCallCheck(this, PostgresqlSyncer);

    this.config = config;
    this.adapter = adapter;
    this.serializer = serializer;
  }

  /**
   * Sync data from the old database to the new one,
   * transforming items to the new version along the way.
   *
   * @param  {String} projectId
   * @param  {Project} oldProject
   * @param  {Project} project
   * @param  {Changes} changes
   * @return {Promise}
   */

  _createClass(PostgresqlSyncer, [{
    key: 'sync',
    value: function sync(projectId, oldProject, project, changes) {
      var _this = this;

      var self, fromConnectionString, fromClient, toClient;
      return regeneratorRuntime.async(function sync$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            debug('-> sync');
            self = this;
            fromConnectionString = 'postgresql://' + this.config.user + ':' + this.config.password + '@' + this.config.host + '/' + oldProject.name + VERSION_SEPERATOR + oldProject.version;
            _context.next = 5;
            return regeneratorRuntime.awrap(this.getConnection(fromConnectionString));

          case 5:
            fromClient = _context.sent;

            fromClient.connection.stream.setMaxListeners(100);
            // retrieve the knex.js client for the new project version
            toClient = this.adapter.getConnection(project.name, project.version);
            return _context.abrupt('return', _bluebird2.default.each(Object.keys(oldProject.schemas), function (schemaId) {
              var oldSchema = oldProject.schemas[schemaId];
              var schema = project.schemas[schemaId];
              if (!schema) {
                return;
              }
              var transformItem = (0, _through2.default)(function (item) {
                Object.keys(oldSchema.columns).forEach(function (columnId) {
                  var oldColumn = oldSchema.columns[columnId];
                  if (self.isGeoField(oldColumn.type)) {
                    item[oldColumn.name] = JSON.parse(item[oldColumn.name]);
                  }
                });
                var transformedItem = _schemaMapperTransformer2.default.transformItem(item, changes);
                if (transformedItem !== null) {
                  this.emit('data', transformedItem);
                }
              });

              var toStream = (0, _through2.default)(function (transformedItem) {
                debug('-> sync.insert');
                var streamSelf = this;
                var serializedItem = self.serializer.serialize(transformedItem, schema);
                streamSelf.pause();
                toClient.table(schema.name).insert(serializedItem)
                // triggers knex.js's promise strategy
                .then(function (res) {
                  streamSelf.resume();
                  return res;
                }).catch(function (err) {
                  throw err;
                });
              });

              return new _bluebird2.default(function (resolve, reject) {
                var query = new _pgQueryStream2.default('SELECT ' + _this.schemaToSelects(oldSchema) + ' FROM "' + oldSchema.name + '"');
                var fromStream = fromClient.query(query);
                fromStream.pipe(transformItem).pipe(toStream).on('end', function () {
                  debug('-> sync.done');
                  resolve();
                }).on('error', function (err) {
                  debug('-> sync.doneWithErrors', err);
                  reject(err);
                });
              });
            }));

          case 9:
          case 'end':
            return _context.stop();
        }
      }, null, this);
    }
  }, {
    key: 'getConnection',
    value: function getConnection(connectionString) {
      var _this2 = this;

      return _pg2.default.connectAsync(connectionString).spread(function (client, done) {
        _this2.connectionCloser = done;
        return client;
      });
    }
  }, {
    key: 'schemaToSelects',
    value: function schemaToSelects(schema) {
      var _this3 = this;

      var selects = ['*'];

      Object.keys(schema.columns).forEach(function (columnId) {
        var column = schema.columns[columnId];

        if (_this3.isGeoField(column.type)) {
          selects.push('ST_AsGeoJson(' + column.name + ') as ' + column.name);
        }

        if (column.type === 'datetime') {
          selects.push('to_char(' + column.name + ', \'yyyy-MM-dd hh:mi:ss\') as ' + column.name);
        }

        if (column.type === 'date') {
          selects.push('to_char(' + column.name + ', \'yyyy-MM-dd\') as ' + column.name);
        }
      });

      return selects.join(',');
    }
  }, {
    key: 'isGeoField',
    value: function isGeoField(type) {
      return (0, _lodash2.default)(['linestring', 'linestring[]', 'point', 'point[]', 'polygon', 'polygon[]'], type);
    }
  }]);

  return PostgresqlSyncer;
}();

exports.default = PostgresqlSyncer;