import pg from 'pg';
import BBPromise from 'bluebird';
BBPromise.promisifyAll(pg);

const VERSION_SEPERATOR = '@';

/**
 * PostgresqlDropper
 *
 * @class PostgresqlDropper
 */
class PostgresqlDropper {
  constructor(config) {
    this.config = config;
  }

  /**
   * Drop the postgresql database.
   *
   * @param  {String} projectName
   * @param  {Number} projectVersion
   *
   * @return {Promise}
   */
  async drop(projectName, projectVersion) {
    let connectionString = 'postgresql://' + this.config.user + ':' + this.config.password + '@' + this.config.host + '/postgres';
    let client = await this.getConnection(connectionString);
    let database = projectName + VERSION_SEPERATOR + String(projectVersion);
    var dropConnectionsResult = await client.queryAsync('SELECT pg_terminate_backend(pid) FROM pg_stat_activity WHERE datname = \'' + database + '\'');
    try {
      var dropDbResult = await client.queryAsync('DROP DATABASE "' + database + '"');
    } catch (e) {
      console.error('If the error says the db didn\'t exist, ignore this', e);
    }
    this.connectionCloser();
  }

  /**
   * Get the connection to the postgres database
   * @protected
   * @param  {String} connectionString
   * @return {Object} The postgresql client
   */
  getConnection(connectionString) {
    return pg.connectAsync(connectionString)
      .spread((client, done) => {
        this.connectionCloser = done;
        return client;
      });
  }

}

export default PostgresqlDropper;
